/***************************************************************************
 *                                                                         *
 *   KCPULoad and KNetLoad are copyright (c) 1999-2000, Markus Gustavsson  *
 *                                       (c) 2002, Ben Burton              *
 *                                       (c) 2004, Diego Pettenò           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STATGRAPH_H
#define STATGRAPH_H

#include <qobject.h>
#include <qstring.h>
#include <qcolor.h>

class QLabel;

/**
 * @author Diego 'Flameeyes' Pettenò
 * 
 * @brief Abstraction of the dock's graph class for KNetLoad and KCPULoad.
 *
 * This classes abstracts the access to the graphs for KNetLoad and KCPULoad
 * (and eventually other apps, too), making possible to use them without
 * system trays and so on.
 *
 * Two simultaneous sets of readings are supported; these will be
 * referred to as upper and lower readings.  When diagram splitting is
 * switched on, these readings will be displayed together on the diagram
 * with the upper readings shown above the lower readings.  When diagram
 * splitting is switched off, only the upper readings will be displayed.
 */
class StatGraph : public QObject {
Q_OBJECT
public:
//@{
/**
 * @name defaultcolors Default graphs' colors
 * @brief These colors will be used as defaults color for graphs.
 */
	static const QColor defaultBgColor;
	static const QColor defaultUpperColor;
	static const QColor defaultLowerColor;
	static const QColor defaultLabelColor;
	static const QColor defaultGridColor;
	static const QColor invalidColor;	///< Color used to pass transparent colors
//@}
	
	enum Style { Lines, Bars, Shades };
protected:
	QLabel *m_graph;	///< Label where the graph will be drawn
	Style m_style;		///< Style of the graph
	QString m_label;	///< Label to show on the graph (QString::null to not show label)
	
	QColor m_bgColor;	///< Background color for the graph @see defaultBgColor
	QColor m_upperColor;	///< Color for the upper reading graph
	QColor m_lowerColor;	///< Color for the lower reading graph
	QColor m_labelColor;	///< Color for the graph's label (if any)
	QColor m_gridColor;	///< Color for the graph's grid (if any)
	
	ushort m_gridPace;	///< Pace at which a the grid is drawn. If 0, the grid is not drawn.
	uint m_readings;	///< Maximum number of readings (label's width / graph's pace)
	uint m_currReading;	///< Current index in the readings arrays.
	uint *m_readingsUpper;	///< Array for upper readings.
	uint *m_readingsLower;	///< Array for lower readings.
public:
	/**
	 * @brief Constructor for a StatGraph instance
	 * @param parent Label where to draw the graph
	 * @param pace X-distance between two reads. This will also influence the number
	 *        of readings which can be added (label's width / pace)
	 * @param gridPace Y-distance between two grid lines. If 0, the grid is not drawn.
	 * @param label Label to show on the graph (QString::null to not show label)
	 * @param bgColor Background color to apply to the graph
	 * @param name Name of the statgraph instance (passed to QObject)
	 *
	 * This constructor initialize the QObject parent class and the
	 * label which will be used to draw the graph on.
	 * Note: it will @b not change the background of the label, nor it will draw
	 * the label. To do this, call redraw() function.
	 */
	StatGraph(QLabel *parent, ushort pace = 1, ushort gridPace = 0, const QString &label = QString::null, const QColor &bgColor = defaultBgColor, const char *name = "");
	
	virtual ~StatGraph();

	/**
	 * @brief Sets the graph's style
	 */
	void setStyle(Style style)
	{ m_style = style; }
	
	/**
	 * @brief Changes the parent of the graph
	 * @param newParent New label to set the parent of the graph
	 * @param pace Pace of the graph, see StatGraph() @c pace parameter.
	 *
	 * This function is used to change the parent of the graph @b and the dimension
	 * of the readings.
	 *
	 * @note When called with @c newParent the same of current parent, it doesn't
	 *       change the relation between the two.
	 * @note The readings will be recreated only if the dimensions are really
	 *       different. No changes will be done if the new parent has the same
	 *       dimensions and pace of the old one.
	 */
	void reparent(QLabel *newParent, ushort pace = 1);
	
private:
	/**
	 * @brief Paints the grid on the graph
	 */
	void paintGrid(QPainter &p);
	
	/**
	 * @brief Paints the graph using Shades style
	 */
	void paintShades(QPainter &p);
	
	/**
	 * @brief Paints the graph using Bars style
	 */
	void paintBars(QPainter &p);
	
	/**
	 * @brief Paints the graph using Lines style
	 */
	void paintLines(QPainter &p);
	
	/**
	 * @brief Paints the label on the graph
	 */
	void paintLabel(QPainter &p);
	
	/**
	 * @brief Apply softening algorithm to readings
	 *
	 * This function is called to 'soften' the readings when addPercentReadings() is
	 * called with @c soft parameter true.
	 */
	void softenReadings(uint &upper, uint &lower);
	
public slots:
	/**
	 * @brief Request a redraw of the graph
	 * 
	 * This function is used to redraw the graph when a setting is changed.
	 * 
	 * @note This function is automatically called when a reading is added to the queue.
	 */
	void redraw();
	
	/**
	 * @brief REquest a clear of the graph
	 *
	 * This functions resets the readings arrays and redraw completely the graph.
	 */
	void clear();
	
	/**
	 * @brief Sets the label to show on the graph
	 *
	 * This function sets the string to show on the graph.
	 * To disable showLabel, simply pass QString::null as label to show.
	 */
	inline void setLabel(const QString &label)
	{
		m_label = label;
	}
	
	/**
	 * @brief Sets the background color for the graph.
	 * 
	 * This function sets the background color for the graph instance
	 * To set the background to transparent, simply pass invalidColor value.
	 */
	void setBgColor(const QColor &color);
	
	/**
	 * @brief Sets the upper reading color for the graph.
	 * 
	 * This function sets the upper reading color for the graph instance.
	 * @note Passing invalidColor to this, made it move to the default color
	 *       as soon as a redraw is requested.
	 */
	inline void setUpperColor(const QColor &color)
	{ m_upperColor = color; }

	/**
	 * @brief Sets the lower reading color for the graph.
	 * 
	 * This function sets the lower reading color for the graph instance.
	 * @note Passing invalidColor to this, made it move to the default color
	 *       as soon as a redraw is requested.
	 */
	inline void setLowerColor(const QColor &color)
	{ m_lowerColor = color; }
	
	/**
	 * @brief Sets the label color for the graph.
	 * 
	 * This function sets the label color for the graph instance.
	 * @note Passing invalidColor to this, made it move to the default color
	 *       as soon as a redraw is requested.
	 */
	inline void setLabelColor(const QColor &color)
	{ m_labelColor = color; }
	
	/**
	 * @brief Sets the grid color for the graph.
	 * 
	 * This function sets the grid color for the graph instance.
	 * @note Passing invalidColor to this, made it move to the default color
	 *       as soon as a redraw is requested.
	 */
	inline void setGridColor(const QColor &color)
	{ m_gridColor = color; }
	
	/**
	 * @brief Sets the grid pace for the graph.
	 * 
	 * This function sets the grid pace for the graph instance.
	 * @note Pass 0 to disable grid painting.
	 */
	inline void setGridPace(const ushort gridpace)
	{ m_gridPace = gridpace; }
	
	/**
	* @brief Add the given pair of readings as the most recent in our list.
	* @param upper The upper reading in this pair.
	* @param lower The lower reading in this pair.
	* @param soft If true, the readings will be aproximated to have a soft curve.
	*
	* Each argument should be a percentage between 0 and 100.
	* The sum of both arguments must not exceed 100.
	*
	* The diagram will be updated accordingly.
	*/
	void addPercentReading(uchar upper, uchar lower, bool soft);
};

#endif
